@@ -1,3 +1,12 @@
+
+"""
+@todo Add params for disable/enable state machines
+@todo Separate voicemail/sms into separate conversation instances
+@todo Setup addressbook, voicemail, sms state machines
+@todo Add option to use screen name as callback
+"""
+
+
 import weakref
 import logging
 
@@ -87,6 +96,10 @@ class TheOneRingConnection(
 	def username(self):
 		return self._credentials[0]
 
+	@property
+	def userAliasType(self):
+		return self.USER_ALIAS_ACCOUNT
+
 	def handle(self, handleType, handleId):
 		self.check_handle(handleType, handleId)
 		return self._handles[handleType, handleId]
@@ -172,24 +185,16 @@ class TheOneRingConnection(
 		self.check_connected()
 		self.check_handle(handleType, handleId)
 
-		channel = None
-		channelManager = self._channelManager
-		handle = self.handle(handleType, handleId)
+		h = self.handle(handleType, handleId) if handleId != 0 else None
+		props = self._generate_props(type, h, suppressHandler)
+		if hasattr(self, "_validate_handle"):
+			# HACK Newer python-telepathy
+			self._validate_handle(props)
 
-		if type == telepathy.CHANNEL_TYPE_CONTACT_LIST:
-			_moduleLogger.info("RequestChannel ContactList")
-			channel = channelManager.channel_for_list(handle, suppressHandler)
-		elif type == telepathy.CHANNEL_TYPE_TEXT:
-			_moduleLogger.info("RequestChannel Text")
-			channel = channelManager.channel_for_text(handle, suppressHandler)
-		elif type == telepathy.CHANNEL_TYPE_STREAMED_MEDIA:
-			_moduleLogger.info("RequestChannel Media")
-			channel = channelManager.channel_for_call(handle, suppressHandler)
-		else:
-			raise telepathy.errors.NotImplemented("unknown channel type %s" % type)
-
-		_moduleLogger.info("RequestChannel Object Path: %s" % channel._object_path)
-		return channel._object_path
+		chan = self._channelManager.channel_for_props(props, signal=True)
+		path = chan._object_path
+		_moduleLogger.info("RequestChannel Object Path: %s" % path)
+		return path
 
 	@gtk_toolbox.log_exception(_moduleLogger)
 	def RequestHandles(self, handleType, names, sender):
@@ -202,26 +207,37 @@ class TheOneRingConnection(
 
 		handles = []
 		for name in names:
-			name = name.encode('utf-8')
+			requestedHandleName = name.encode('utf-8')
 			if handleType == telepathy.HANDLE_TYPE_CONTACT:
-				_moduleLogger.info("RequestHandles Contact: %s" % name)
-				h = self._create_contact_handle(name)
+				_moduleLogger.info("RequestHandles Contact: %s" % requestedHandleName)
+				requestedContactId, requestedContactNumber = handle.ContactHandle.from_handle_name(
+					requestedHandleName
+				)
+				h = handle.create_handle(self, 'contact', requestedContactId, requestedContactNumber)
 			elif handleType == telepathy.HANDLE_TYPE_LIST:
 				# Support only server side (immutable) lists
-				_moduleLogger.info("RequestHandles List: %s" % name)
-				h = handle.create_handle(self, 'list', name)
+				_moduleLogger.info("RequestHandles List: %s" % requestedHandleName)
+				h = handle.create_handle(self, 'list', requestedHandleName)
 			else:
 				raise telepathy.errors.NotAvailable('Handle type unsupported %d' % handleType)
 			handles.append(h.id)
 			self.add_client_handle(h, sender)
 		return handles
 
-	def _create_contact_handle(self, requestedHandleName):
-		requestedContactId, requestedContactNumber = handle.ContactHandle.from_handle_name(
-			requestedHandleName
-		)
-		h = handle.create_handle(self, 'contact', requestedContactId, requestedContactNumber)
-		return h
+	def _generate_props(self, channelType, handle, suppressHandler, initiatorHandle=None):
+		targetHandle = 0 if handle is None else handle.get_id()
+		targetHandleType = telepathy.HANDLE_TYPE_NONE if handle is None else handle.get_type()
+		props = {
+			telepathy.CHANNEL_INTERFACE + '.ChannelType': channelType,
+			telepathy.CHANNEL_INTERFACE + '.TargetHandle': targetHandle,
+			telepathy.CHANNEL_INTERFACE + '.TargetHandleType': targetHandleType,
+			telepathy.CHANNEL_INTERFACE + '.Requested': suppressHandler
+		}
+
+		if initiatorHandle is not None:
+			props[telepathy.CHANNEL_INTERFACE + '.InitiatorHandle'] = initiatorHandle.id
+
+		return props
 
 	@gobject_utils.async
 	@gtk_toolbox.log_exception(_moduleLogger)
@@ -229,8 +245,8 @@ class TheOneRingConnection(
 		# @todo get conversations update running
 		# @todo test conversatiuons
 		_moduleLogger.info("Incoming messages from: %r" % (conversationIds, ))
-		channelManager = self._channelManager
 		for contactId, phoneNumber in conversationIds:
 			h = handle.create_handle(self, 'contact', contactId, phoneNumber)
 			# Just let the TextChannel decide whether it should be reported to the user or not
-			channel = channelManager.channel_for_text(h)
+			props = self._generate_props(telepathy.CHANNEL_TYPE_TEXT, h, False)
+			channel = self._channelManager.channel_for_props(props, signal=True)
