@@ -37,9 +37,9 @@ class StateMachine(object):
 	_ACTION_RESET = "reset"
 	_ACTION_STOP = "stop"
 
-	_INITIAL_ACTIVE_PERIOD = int(_to_milliseconds(seconds=5))
-	_FINAL_ACTIVE_PERIOD = int(_to_milliseconds(minutes=2))
-	_IDLE_PERIOD = int(_to_milliseconds(minutes=10))
+	_INITIAL_ACTIVE_PERIOD = int(_to_milliseconds(seconds=10))
+	_FINAL_ACTIVE_PERIOD = int(_to_milliseconds(minutes=10))
+	_IDLE_PERIOD = int(_to_milliseconds(minutes=30))
 	_INFINITE_PERIOD = -1
 
 	_IS_DAEMON = True
@@ -49,6 +49,7 @@ class StateMachine(object):
 		self._updateItems = updateItems
 
 		self._state = self.STATE_ACTIVE
+		self._startId = None
 		self._timeoutId = None
 		self._currentPeriod = self._INITIAL_ACTIVE_PERIOD
 		self._set_initial_period()
@@ -62,20 +63,16 @@ class StateMachine(object):
 	def close(self):
 		self._callback = None
 
-	@gobject_utils.async
-	@gtk_toolbox.log_exception(_moduleLogger)
 	def start(self):
-		_moduleLogger.info("Starting State Machine")
-		for item in self._initItems:
-			try:
-				item.update()
-			except Exception:
-				_moduleLogger.exception("Initial update failed for %r" % item)
-		self._schedule_update()
-		return False # do not continue
+		assert self._startId is None
+		assert self._timeoutId is None
+		self._startId = gobject.idle_add(self._start)
 
 	def stop(self):
-		_moduleLogger.info("Stopping an already stopped state machine")
+		if self._startId is not None:
+			_moduleLogger.info("Stopping state machine before it even had a chance to start")
+			gobject.source_remove(self._startId)
+			self._startId = None
 		self._stop_update()
 
 	def set_state(self, newState):
@@ -83,7 +80,7 @@ class StateMachine(object):
 		_moduleLogger.info("Transitioning from %s to %s" % (oldState, newState))
 
 		self._state = newState
-		self.reset_timers()
+		self._reset_timers()
 
 	def get_state(self):
 		return self._state
@@ -98,7 +95,7 @@ class StateMachine(object):
 	@gobject_utils.async
 	@gtk_toolbox.log_exception(_moduleLogger)
 	def _request_reset_timers(self, *args):
-		self.reset_timers()
+		self._reset_timers()
 
 	def _set_initial_period(self):
 		self._currentPeriod = self._INITIAL_ACTIVE_PERIOD / 2 # We will double it later
@@ -112,8 +109,20 @@ class StateMachine(object):
 		_moduleLogger.info("Next update in %s ms" % (nextTimeout, ))
 		self._currentPeriod = nextTimeout
 
+	def _start(self):
+		_moduleLogger.info("Starting State Machine")
+		for item in self._initItems:
+			try:
+				item.update()
+			except Exception:
+				_moduleLogger.exception("Initial update failed for %r" % item)
+		self._schedule_update()
+		self._startId = None
+		return False # do not continue
+
 	def _stop_update(self):
 		if self._timeoutId is None:
+			_moduleLogger.info("Stopping an already stopped state machine")
 			return
 		gobject.source_remove(self._timeoutId)
 		self._timeoutId = None
